@@ -83,6 +83,8 @@ def _get_running_interactive_framework():
                 if frame.f_code in codes:
                     return "tk"
                 frame = frame.f_back
+        # premetively break reference cycle between locals and the frame
+        del frame
     macosx = sys.modules.get("matplotlib.backends._macosx")
     if macosx and macosx.event_loop_is_running():
         return "macosx"
@@ -132,9 +134,9 @@ class CallbackRegistry:
     for a set of signals and callbacks:
 
         >>> def oneat(x):
-        ...    print('eat', x)
+        ...     print('eat', x)
         >>> def ondrink(x):
-        ...    print('drink', x)
+        ...     print('drink', x)
 
         >>> from matplotlib.cbook import CallbackRegistry
         >>> callbacks = CallbackRegistry()
@@ -206,9 +208,11 @@ class CallbackRegistry:
                           for s, d in self.callbacks.items()},
             # It is simpler to reconstruct this from callbacks in __setstate__.
             "_func_cid_map": None,
+            "_cid_gen": next(self._cid_gen)
         }
 
     def __setstate__(self, state):
+        cid_count = state.pop('_cid_gen')
         vars(self).update(state)
         self.callbacks = {
             s: {cid: _weak_or_strong_ref(func, self._remove_proxy)
@@ -217,12 +221,10 @@ class CallbackRegistry:
         self._func_cid_map = {
             s: {proxy: cid for cid, proxy in d.items()}
             for s, d in self.callbacks.items()}
+        self._cid_gen = itertools.count(cid_count)
 
     def connect(self, signal, func):
         """Register *func* to be called when signal *signal* is generated."""
-        if signal == "units finalize":
-            _api.warn_deprecated(
-                "3.5", name=signal, obj_type="signal", alternative="units")
         if self._signals is not None:
             _api.check_in_list(self._signals, signal=signal)
         self._func_cid_map.setdefault(signal, {})
@@ -708,37 +710,6 @@ class Stack:
                 self.push(elem)
 
 
-@_api.deprecated("3.5", alternative="psutil.virtual_memory")
-def report_memory(i=0):  # argument may go away
-    """Return the memory consumed by the process."""
-    def call(command, os_name):
-        try:
-            return subprocess.check_output(command)
-        except subprocess.CalledProcessError as err:
-            raise NotImplementedError(
-                "report_memory works on %s only if "
-                "the '%s' program is found" % (os_name, command[0])
-            ) from err
-
-    pid = os.getpid()
-    if sys.platform == 'sunos5':
-        lines = call(['ps', '-p', '%d' % pid, '-o', 'osz'], 'Sun OS')
-        mem = int(lines[-1].strip())
-    elif sys.platform == 'linux':
-        lines = call(['ps', '-p', '%d' % pid, '-o', 'rss,sz'], 'Linux')
-        mem = int(lines[1].split()[1])
-    elif sys.platform == 'darwin':
-        lines = call(['ps', '-p', '%d' % pid, '-o', 'rss,vsz'], 'Mac OS')
-        mem = int(lines[1].split()[0])
-    elif sys.platform == 'win32':
-        lines = call(["tasklist", "/nh", "/fi", "pid eq %d" % pid], 'Windows')
-        mem = int(lines.strip().split()[-2].replace(',', ''))
-    else:
-        raise NotImplementedError(
-            "We don't have a memory monitor for %s" % sys.platform)
-    return mem
-
-
 def safe_masked_invalid(x, copy=False):
     x = np.array(x, subok=True, copy=copy)
     if not x.dtype.isnative:
@@ -1509,7 +1480,7 @@ def violin_stats(X, method, points=100, quantiles=None):
     else:
         quantiles = [[]] * len(X)
 
-    # quantiles should has the same size as dataset
+    # quantiles should have the same size as dataset
     if len(X) != len(quantiles):
         raise ValueError("List of violinplot statistics and quantiles values"
                          " must have the same length")
@@ -1700,7 +1671,7 @@ def safe_first_element(obj):
     """
     Return the first element in *obj*.
 
-    This is an type-independent way of obtaining the first element,
+    This is a type-independent way of obtaining the first element,
     supporting both index access and the iterator protocol.
     """
     return _safe_first_finite(obj, skip_nonfinite=False)
@@ -1708,13 +1679,13 @@ def safe_first_element(obj):
 
 def _safe_first_finite(obj, *, skip_nonfinite=True):
     """
-    Return the first non-None (and optionally finite) element in *obj*.
+    Return the first finite element in *obj* if one is available and skip_nonfinite is
+    True. Otherwise return the first element.
 
     This is a method for internal use.
 
-    This is an type-independent way of obtaining the first non-None element,
-    supporting both index access and the iterator protocol.
-    The first non-None element will be obtained when skip_none is True.
+    This is a type-independent way of obtaining the first finite element, supporting
+    both index access and the iterator protocol.
     """
     def safe_isfinite(val):
         if val is None:
@@ -1746,7 +1717,7 @@ def _safe_first_finite(obj, *, skip_nonfinite=True):
         raise RuntimeError("matplotlib does not "
                            "support generators as input")
     else:
-        return next(val for val in obj if safe_isfinite(val))
+        return next((val for val in obj if safe_isfinite(val)), safe_first_element(obj))
 
 
 def sanitize_sequence(data):
@@ -1905,7 +1876,7 @@ def _array_perimeter(arr):
 
     Examples
     --------
-    >>> i, j = np.ogrid[:3,:4]
+    >>> i, j = np.ogrid[:3, :4]
     >>> a = i*10 + j
     >>> a
     array([[ 0,  1,  2,  3],
@@ -1949,7 +1920,7 @@ def _unfold(arr, axis, size, step):
 
     Examples
     --------
-    >>> i, j = np.ogrid[:3,:7]
+    >>> i, j = np.ogrid[:3, :7]
     >>> a = i*10 + j
     >>> a
     array([[ 0,  1,  2,  3,  4,  5,  6],
@@ -2090,7 +2061,7 @@ class _OrderedSet(collections.abc.MutableSet):
         self._od.pop(key, None)
 
 
-# Agg's buffers are unmultiplied RGBA8888, which neither PyQt5 nor cairo
+# Agg's buffers are unmultiplied RGBA8888, which neither PyQt<=5.1 nor cairo
 # support; however, both do support premultiplied ARGB32.
 
 
@@ -2336,3 +2307,30 @@ def _unpack_to_numpy(x):
         if isinstance(xtmp, np.ndarray):
             return xtmp
     return x
+
+
+def _auto_format_str(fmt, value):
+    """
+    Apply *value* to the format string *fmt*.
+
+    This works both with unnamed %-style formatting and
+    unnamed {}-style formatting. %-style formatting has priority.
+    If *fmt* is %-style formattable that will be used. Otherwise,
+    {}-formatting is applied. Strings without formatting placeholders
+    are passed through as is.
+
+    Examples
+    --------
+    >>> _auto_format_str('%.2f m', 0.2)
+    '0.20 m'
+    >>> _auto_format_str('{} m', 0.2)
+    '0.2 m'
+    >>> _auto_format_str('const', 0.2)
+    'const'
+    >>> _auto_format_str('%d or {}', 0.2)
+    '0 or {}'
+    """
+    try:
+        return fmt % (value,)
+    except (TypeError, ValueError):
+        return fmt.format(value)
