@@ -7,11 +7,11 @@ from types import SimpleNamespace
 
 import numpy as np
 
+import matplotlib as mpl
 from matplotlib.path import Path
 from matplotlib.patches import PathPatch
 from matplotlib.transforms import Affine2D
-from matplotlib import docstring
-from matplotlib import rcParams
+from matplotlib import _docstring
 
 _log = logging.getLogger(__name__)
 
@@ -45,50 +45,9 @@ class Sankey:
         """
         Create a new Sankey instance.
 
-        Optional keyword arguments:
-
-          ===============   ===================================================
-          Field             Description
-          ===============   ===================================================
-          *ax*              axes onto which the data should be plotted
-                            If *ax* isn't provided, new axes will be created.
-          *scale*           scaling factor for the flows
-                            *scale* sizes the width of the paths in order to
-                            maintain proper layout.  The same scale is applied
-                            to all subdiagrams.  The value should be chosen
-                            such that the product of the scale and the sum of
-                            the inputs is approximately 1.0 (and the product of
-                            the scale and the sum of the outputs is
-                            approximately -1.0).
-          *unit*            string representing the physical unit associated
-                            with the flow quantities
-                            If *unit* is None, then none of the quantities are
-                            labeled.
-          *format*          a Python number formatting string to be used in
-                            labeling the flow as a quantity (i.e., a number
-                            times a unit, where the unit is given)
-          *gap*             space between paths that break in/break away
-                            to/from the top or bottom
-          *radius*          inner radius of the vertical paths
-          *shoulder*        size of the shoulders of output arrowS
-          *offset*          text offset (from the dip or tip of the arrow)
-          *head_angle*      angle of the arrow heads (and negative of the angle
-                            of the tails) [deg]
-          *margin*          minimum space between Sankey outlines and the edge
-                            of the plot area
-          *tolerance*       acceptable maximum of the magnitude of the sum of
-                            flows
-                            The magnitude of the sum of connected flows cannot
-                            be greater than *tolerance*.
-          ===============   ===================================================
-
-        The optional arguments listed above are applied to all subdiagrams so
+        The optional arguments listed below are applied to all subdiagrams so
         that there is consistent alignment and formatting.
 
-        If :class:`Sankey` is instantiated with any keyword arguments other
-        than those explicitly listed above (``**kwargs``), they will be passed
-        to :meth:`add`, which will create the first subdiagram.
-
         In order to draw a complex Sankey diagram, create an instance of
         :class:`Sankey` by calling it without any kwargs::
 
@@ -109,6 +68,50 @@ class Sankey:
 
             Sankey().add().add...  .add().finish()
 
+        Other Parameters
+        ----------------
+        ax : `~matplotlib.axes.Axes`
+            Axes onto which the data should be plotted.  If *ax* isn't
+            provided, new Axes will be created.
+        scale : float
+            Scaling factor for the flows.  *scale* sizes the width of the paths
+            in order to maintain proper layout.  The same scale is applied to
+            all subdiagrams.  The value should be chosen such that the product
+            of the scale and the sum of the inputs is approximately 1.0 (and
+            the product of the scale and the sum of the outputs is
+            approximately -1.0).
+        unit : str
+            The physical unit associated with the flow quantities.  If *unit*
+            is None, then none of the quantities are labeled.
+        format : str or callable
+            A Python number formatting string or callable used to label the
+            flows with their quantities (i.e., a number times a unit, where the
+            unit is given). If a format string is given, the label will be
+            ``format % quantity``. If a callable is given, it will be called
+            with ``quantity`` as an argument.
+        gap : float
+            Space between paths that break in/break away to/from the top or
+            bottom.
+        radius : float
+            Inner radius of the vertical paths.
+        shoulder : float
+            Size of the shoulders of output arrows.
+        offset : float
+            Text offset (from the dip or tip of the arrow).
+        head_angle : float
+            Angle, in degrees, of the arrow heads (and negative of the angle of
+            the tails).
+        margin : float
+            Minimum space between Sankey outlines and the edge of the plot
+            area.
+        tolerance : float
+            Acceptable maximum of the magnitude of the sum of flows.  The
+            magnitude of the sum of connected flows cannot be greater than
+            *tolerance*.
+        **kwargs
+            Any additional keyword arguments will be passed to :meth:`add`,
+            which will create the first subdiagram.
+
         See Also
         --------
         Sankey.add
@@ -168,15 +171,17 @@ class Sankey:
         Return the codes and vertices for a rotated, scaled, and translated
         90 degree arc.
 
-        Optional keyword arguments:
-
-          ===============   ==========================================
-          Keyword           Description
-          ===============   ==========================================
-          *quadrant*        uses 0-based indexing (0, 1, 2, or 3)
-          *cw*              if True, clockwise
-          *center*          (x, y) tuple of the arc's center
-          ===============   ==========================================
+        Other Parameters
+        ----------------
+        quadrant : {0, 1, 2, 3}, default: 0
+            Uses 0-based indexing (0, 1, 2, or 3).
+        cw : bool, default: True
+            If True, the arc vertices are produced clockwise; counter-clockwise
+            otherwise.
+        radius : float, default: 1
+            The radius of the arc.
+        center : (float, float), default: (0, 0)
+            (x, y) tuple of the arc's center.
         """
         # Note:  It would be possible to use matplotlib's transforms to rotate,
         # scale, and translate the arc, but since the angles are discrete,
@@ -199,12 +204,12 @@ class Sankey:
                                  # Insignificant
                                  # [6.12303177e-17, 1.00000000e+00]])
                                  [0.00000000e+00, 1.00000000e+00]])
-        if quadrant == 0 or quadrant == 2:
+        if quadrant in (0, 2):
             if cw:
                 vertices = ARC_VERTICES
             else:
                 vertices = ARC_VERTICES[:, ::-1]  # Swap x and y.
-        elif quadrant == 1 or quadrant == 3:
+        else:  # 1, 3
             # Negate x.
             if cw:
                 # Swap x and y.
@@ -294,15 +299,11 @@ class Sankey:
             else:  # Vertical
                 x += self.gap
                 if angle == UP:
-                    sign = 1
+                    sign, quadrant = 1, 3
                 else:
-                    sign = -1
+                    sign, quadrant = -1, 0
 
                 tip = [x - flow / 2.0, y + sign * (length + tipheight)]
-                if angle == UP:
-                    quadrant = 3
-                else:
-                    quadrant = 0
                 # Inner arc isn't needed if inner radius is zero
                 if self.radius:
                     path.extend(self._arc(quadrant=quadrant,
@@ -346,7 +347,7 @@ class Sankey:
         # path[2] = path[2][::-1]
         # return path
 
-    @docstring.dedent_interpd
+    @_docstring.dedent_interpd
     def add(self, patchlabel='', flows=None, orientations=None, labels='',
             trunklength=1.0, pathlengths=0.25, prior=None, connect=(0, 0),
             rotation=0, **kwargs):
@@ -370,7 +371,7 @@ class Sankey:
             the outside in.
 
             If the sum of the inputs and outputs is
-            nonzero, the discrepancy will appear as a cubic Bezier curve along
+            nonzero, the discrepancy will appear as a cubic Bézier curve along
             the top and bottom edges of the trunk.
 
         orientations : list of {-1, 0, 1}
@@ -426,17 +427,14 @@ class Sankey:
            properties, listed below.  For example, one may want to use
            ``fill=False`` or ``label="A legend entry"``.
 
-        %(Patch)s
+        %(Patch:kwdoc)s
 
         See Also
         --------
         Sankey.finish
         """
         # Check and preprocess the arguments.
-        if flows is None:
-            flows = np.array([1.0, -1.0])
-        else:
-            flows = np.array(flows)
+        flows = np.array([1.0, -1.0]) if flows is None else np.array(flows)
         n = flows.shape[0]  # Number of flows
         if rotation is None:
             rotation = 0
@@ -463,7 +461,7 @@ class Sankey:
             raise ValueError(
                 "'trunklength' is negative, which is not allowed because it "
                 "would cause poor layout")
-        if np.abs(np.sum(flows)) > self.tolerance:
+        if abs(np.sum(flows)) > self.tolerance:
             _log.info("The sum of the flows is nonzero (%f; patchlabel=%r); "
                       "is the system not at steady state?",
                       np.sum(flows), patchlabel)
@@ -520,7 +518,7 @@ class Sankey:
             if orient == 1:
                 if is_input:
                     angles[i] = DOWN
-                elif not is_input:
+                elif is_input is False:
                     # Be specific since is_input can be None.
                     angles[i] = UP
             elif orient == 0:
@@ -533,7 +531,7 @@ class Sankey:
                         f"but it must be -1, 0, or 1")
                 if is_input:
                     angles[i] = UP
-                elif not is_input:
+                elif is_input is False:
                     angles[i] = DOWN
 
         # Justify the lengths of the paths.
@@ -556,7 +554,7 @@ class Sankey:
                 if angle == DOWN and is_input:
                     pathlengths[i] = ullength
                     ullength += flow
-                elif angle == UP and not is_input:
+                elif angle == UP and is_input is False:
                     pathlengths[i] = urlength
                     urlength -= flow  # Flow is negative for outputs.
             # Determine the lengths of the bottom-side arrows
@@ -566,7 +564,7 @@ class Sankey:
                 if angle == UP and is_input:
                     pathlengths[n - i - 1] = lllength
                     lllength += flow
-                elif angle == DOWN and not is_input:
+                elif angle == DOWN and is_input is False:
                     pathlengths[n - i - 1] = lrlength
                     lrlength -= flow
             # Determine the lengths of the left-side arrows
@@ -586,7 +584,7 @@ class Sankey:
             for i, (angle, is_input, spec) in enumerate(zip(
                   angles, are_inputs, list(zip(scaled_flows, pathlengths)))):
                 if angle == RIGHT:
-                    if not is_input:
+                    if is_input is False:
                         if has_right_output:
                             pathlengths[i] = 0
                         else:
@@ -632,7 +630,7 @@ class Sankey:
             if angle == DOWN and is_input:
                 tips[i, :], label_locations[i, :] = self._add_input(
                     ulpath, angle, *spec)
-            elif angle == UP and not is_input:
+            elif angle == UP and is_input is False:
                 tips[i, :], label_locations[i, :] = self._add_output(
                     urpath, angle, *spec)
         # Add the bottom-side inputs and outputs from the middle outwards.
@@ -642,7 +640,7 @@ class Sankey:
                 tip, label_location = self._add_input(llpath, angle, *spec)
                 tips[n - i - 1, :] = tip
                 label_locations[n - i - 1, :] = label_location
-            elif angle == DOWN and not is_input:
+            elif angle == DOWN and is_input is False:
                 tip, label_location = self._add_output(lrpath, angle, *spec)
                 tips[n - i - 1, :] = tip
                 label_locations[n - i - 1, :] = label_location
@@ -665,7 +663,7 @@ class Sankey:
         has_right_output = False
         for i, (angle, is_input, spec) in enumerate(zip(
               angles, are_inputs, list(zip(scaled_flows, pathlengths)))):
-            if angle == RIGHT and not is_input:
+            if angle == RIGHT and is_input is False:
                 if not has_right_output:
                     # Make sure the upper path extends
                     # at least as far as the lower one.
@@ -718,7 +716,7 @@ class Sankey:
             vertices = translate(rotate(vertices))
             kwds = dict(s=patchlabel, ha='center', va='center')
             text = self.ax.text(*offset, **kwds)
-        if rcParams['_internal.classic_mode']:
+        if mpl.rcParams['_internal.classic_mode']:
             fc = kwargs.pop('fc', kwargs.pop('facecolor', '#bfd1d4'))
             lw = kwargs.pop('lw', kwargs.pop('linewidth', 0.5))
         else:
@@ -736,7 +734,13 @@ class Sankey:
             if label is None or angle is None:
                 label = ''
             elif self.unit is not None:
-                quantity = self.format % abs(number) + self.unit
+                if isinstance(self.format, str):
+                    quantity = self.format % abs(number) + self.unit
+                elif callable(self.format):
+                    quantity = self.format(number)
+                else:
+                    raise TypeError(
+                        'format must be callable or a format string')
                 if label != '':
                     label += "\n"
                 label += quantity
